@@ -6,7 +6,7 @@
 ;; Maintainer: Protesilaos Stavrou General Issues <~protesilaos/general-issues@lists.sr.ht>
 ;; URL: https://git.sr.ht/~protesilaos/spacious-padding
 ;; Mailing-List: https://lists.sr.ht/~protesilaos/general-issues
-;; Version: 0.2.2
+;; Version: 0.3.0
 ;; Package-Requires: ((emacs "28.1"))
 ;; Keywords: convenience, focus, writing, presentation
 
@@ -31,7 +31,9 @@
 ;; spacing/padding of Emacs windows and frames.  The idea is to make
 ;; editing and reading feel more comfortable.  Enable the
 ;; `spacious-padding-mode'.  Adjust the exact spacing values by
-;; modifying the user option `spacious-padding-widths'.
+;; modifying the user option `spacious-padding-widths'.  To have a
+;; more subtle mode line, with nothing but an overline, configure the
+;; user option `spacious-padding-subtle-mode-line'.
 ;;
 ;; Inspiration for this package comes from Nicolas Rougier's
 ;; impressive designs[1] and Daniel Mendler's `org-modern`
@@ -88,8 +90,76 @@ following:
   :package-version '(spacious-padding . "0.2.0")
   :group 'spacious-padding)
 
+(defcustom spacious-padding-subtle-mode-line nil
+  "Remove the background from the mode lines and add overlines.
+
+Preserve whatever padding is specified in `spacious-padding-widths'.
+
+The value is either a boolean type or a plist.  If it is non-nil,
+use the foreground of the underlying mode line face to derive the
+color of the overline.
+
+If the non-nil value is a plist read the following keys to
+determine the exact style of the overlines.
+
+- `:mode-line-active' refers to the active/current mode line.
+
+- `:mode-line-inactive' refers to the inactive/non-current mode
+  lines.
+
+Each key accepts either a face or a string representing a color
+as its associated value:
+
+- The face is an unquoted symbol, such as `success' or `shadow',
+  whose `:foreground' attribute is queried to extract the desired
+  color value.
+
+- The color is a name among those listed in the output of the
+  command `list-colors-display' or a hexadecimal RGB value, such
+  as #123456.
+
+If the key is missing or its value is not one of the above, fall
+back to reading the foreground of the underlying mode line face
+to determine the color of the overline.
+
+Examples of valid configurations:
+
+    ;; Use the foreground of the underlying face to determine the color of
+    ;; the overline (e.g. the inactive mode line has gray text, so render
+    ;; the overline in the same gray).
+    (setq spacious-padding-subtle-mode-line t)
+
+    ;; Use the foreground of the `error' face (typically a red hue) for
+    ;; the active mode line's overline.  For the inactive mode line, fall
+    ;; back to the foreground color of the underlying face (as in the case
+    ;; of the t shown above).
+    (setq spacious-padding-subtle-mode-line
+          \\='(:mode-line-active error))
+
+    ;; As above, but now use the foreground of the `shadow' face for the
+    ;; inactive mode line.
+    (setq spacious-padding-subtle-mode-line
+          \\='(:mode-line-active error :mode-line-inactive shadow))
+
+    ;; Use color values directly.
+    (setq spacious-padding-subtle-mode-line
+          \\='(:mode-line-active \"#0000ff\" :mode-line-inactive \"gray50\"))"
+  :type '(choice boolean
+                 (plist
+                  :key-type (choice (const :mode-line-active)
+                                    (const :mode-line-inactive))
+                  :value-type (choice string face)))
+  :package-version '(spacious-padding . "0.3.0")
+  :group 'spacious-padding)
+
+;; NOTE 2023-12-05: The `keycast-key' should preferably be
+;; disambiguated into separate faces for all the places where keycast
+;; can be displayed (mode line, header line, tab bar).  For now I am
+;; treating it as a mode line face which means that the mode line
+;; padding will be applied elsewhere if keycast is shown there.  Not a
+;; huge problem, but I am aware of it.
 (defvar spacious-padding--mode-line-faces
-  '(mode-line mode-line-active mode-line-inactive mode-line-highlight)
+  '(mode-line mode-line-active mode-line-inactive mode-line-highlight keycast-key)
   "Mode line faces relevant to `spacious-padding-mode'.")
 
 (defvar spacious-padding--header-line-faces
@@ -117,14 +187,35 @@ Return 4 if KEY does not have a value."
     (spacious-padding--get-box-width :tab-width))
    (t (error "`%s' is not relevant to `spacious-padding-mode'" face))))
 
-(defun spacious-padding-set-face-box-padding (face fallback)
-  "Return appropriate face attributes for FACE with FALLBACK face background."
+(defun spacious-padding--get-face-overline-color (face fallback subtle-key)
+  "Get overline foreground value for FACE with FALLBACK face if needed.
+Use SUBTLE-KEY to determine the value based on
+`spacious-padding-subtle-mode-line', falling back to FACE, then
+FALLBACK."
+  (let ((subtle-value (plist-get spacious-padding-subtle-mode-line subtle-key)))
+    (cond
+     ((stringp subtle-value) subtle-value)
+     ((facep subtle-value) (face-foreground subtle-value nil face))
+     (t (face-foreground face nil fallback)))))
+
+(defun spacious-padding-set-face-box-padding (face fallback &optional subtle-key)
+  "Return face attributes for FACE with FALLBACK face background.
+With optional SUBTLE-KEY, read its value from the
+`spacious-padding-subtle-mode-line' and apply it to FACE as an
+overline."
   (when (facep face)
-    (list :box
-          (list
-           :line-width (spacious-padding--get-face-width face)
-           :color (face-background face nil fallback)
-           :style nil))))
+    (let* ((original-bg (face-background face nil fallback))
+           (subtle-bg (face-background 'default))
+           (subtlep (and subtle-key spacious-padding-subtle-mode-line))
+           (bg (if subtlep subtle-bg original-bg)))
+      `(,@(when subtlep
+            (list
+             :background bg
+             :overline (spacious-padding--get-face-overline-color face fallback subtle-key)))
+        :box
+        ( :line-width ,(spacious-padding--get-face-width face)
+          :color ,bg
+          :style nil)))))
 
 (defun spacious-padding-set-invisible-dividers (_theme)
   "Make window dividers for THEME invisible."
@@ -135,10 +226,11 @@ Return 4 if KEY does not have a value."
      `(line-number ((t :background ,bg-main)))
      `(header-line ((t ,@(spacious-padding-set-face-box-padding 'header-line 'default))))
      `(header-line-highlight ((t :box (:color ,fg-main))))
-     `(mode-line ((t ,@(spacious-padding-set-face-box-padding 'mode-line 'default))))
+     `(keycast-key ((t ,@(spacious-padding-set-face-box-padding 'keycast-key 'default))))
+     `(mode-line ((t ,@(spacious-padding-set-face-box-padding 'mode-line 'default :mode-line-active))))
      ;; We cannot use :inherit mode-line because it does not get our version of it...
-     `(mode-line-active ((t ,@(spacious-padding-set-face-box-padding 'mode-line-active 'mode-line))))
-     `(mode-line-inactive ((t ,@(spacious-padding-set-face-box-padding 'mode-line-inactive 'mode-line))))
+     `(mode-line-active ((t ,@(spacious-padding-set-face-box-padding 'mode-line-active 'mode-line :mode-line-active))))
+     `(mode-line-inactive ((t ,@(spacious-padding-set-face-box-padding 'mode-line-inactive 'mode-line :mode-line-inactive))))
      `(mode-line-highlight ((t :box (:color ,fg-main))))
      `(tab-bar-tab ((t ,@(spacious-padding-set-face-box-padding 'tab-bar-tab 'tab-bar))))
      `(tab-bar-tab-inactive ((t ,@(spacious-padding-set-face-box-padding 'tab-bar-tab-inactive 'tab-bar))))
@@ -152,6 +244,7 @@ Return 4 if KEY does not have a value."
    '(fringe (( )))
    '(line-number (( )))
    '(header-line (( )))
+   '(keycast-key (( )))
    '(header-line-highlight (( )))
    '(mode-line (( )))
    '(mode-line-active (( )))
